 /*
BMP180 library reads the BMP180 sensor via i2c, 
   BMP180 returns temperature T (+-0.5 °C), pressure p (+-0.02 to 0.06 mbar).
SHT21 library reads the SHT21 sensor via i2c, 
   SHT21 returns temperature T (+-0.5 °C) and humidity H (+-3-5%).
Real time clock (RTC) gives time since Jan 1, 1971 in seconds via i2c.
Data is written to Logfile on SD card into numbered pTH_xxxx.log files 
*/

#define SensorNumber 3      // Number the sensors to keep track of data

#include <Arduino.h> 
#include <SD.h>              // SD card library
#include <Bmp180n.h>         // BMP180 pressure  and temperature sensor
#include <SHT21.h>           // SHT15 humidity and temperature sensor
#include <RTC_I2C.h>         // Real-Time-Clock DS1307

#define LOG_INTERVAL  5000   // Pause between log entries
#define ECHO_TO_SERIAL   1   // Echo data to serial port?
#define greenLEDpin 3        // Digital pin for green status LED
#define redLEDpin 4          // Digital pin for red status LED
#define CDpin 9              // Pin connected to Card Detect switch (CD)
#define chipSelect 10        // ChipSelect pin (10 for Adafruit card), pins 11,12,13 are also used for SD
SHT21 SHT(A4,A5);            // SHT21  I2C communication (SDA pin, SCL pin)
Bmp180 BMP(A4,A5,3);         // Bmp180 I2C communication (sda-pin,scl-pin,oversampling) 
RTC rtc(A4,A5);              // DS1307 I2C communication (sda-pin,scl-pin)

boolean logging = 0;
char filename[] = "pTH_0000.log";      // Log filename 
File logFile;                          // LogFile identifier

/************ Initialization **************/
void setup()
{
  pinMode(redLEDpin, OUTPUT);           // Initialization: light both LEDs
  pinMode(greenLEDpin, OUTPUT);         //
  pinMode(chipSelect, OUTPUT);          // Chip select pin must be set to output
  pinMode(CDpin, INPUT_PULLUP);         // SD card detect pin must be pulled high
  digitalWrite(greenLEDpin, LOW);      //
  digitalWrite(redLEDpin, LOW);        //
   
  Serial.begin (19200);                 // Set up the Serial Connection.
  BMP.Bmp180_init();                    // Load calibration data friom BMP180
  Logfile_init();
}

void loop() 
{
  static int32_t T,p;                       // Variables for BMP180 pressure and  temperature
  float H,T2;                               // Variables for SHT15 temperature and humidity             

   SHT.SHT21_HT(&H, &T2);                    // Read SHT15 temperature and humidity
  delay(200);                               // Delay between SHT and BMP reading gives more accurate values

  BMP.Bmp180_pT(&p,&T);                     // Read pressure in Pa, temperature in 0.1(deg C)
  delay(50);                                // Delay between SHT and BMP reading gives more accurate values
  
  rtc.timeNow();    // read time            // Read time
  delay(50);                                // Delay to ensure there is power for SD write
  
  Log("\r\n" + rtc.hourString(),0, logFile);  // Write hours (fractional) into log file
  Log(",",T2,logFile);                   // Write log string into log file
  Log(",",H,logFile);                      // Write log string into log file
  Log(",",float(T)/10,logFile);            // Write log string into log file
  Log(",",float(p)/100,logFile);           // Write log string into log file
  Log(",  " + rtc.timeString(),0, logFile);  // Write time into log file
  
  delay(LOG_INTERVAL);                      // Delay between sensor readouts

} 

/******* Check if SD Card is inserted *******/
byte SD_present()
{
  if (!digitalRead(CDpin)) {              // If SD card is inserted (CDpin 0: card inserted)
    if (!logging){                        // ... but logging didn't start
      Logfile_init(); }                   // ... ... then start logging
    return logging; }                     // ... return 1 if the card is inserted and initialized
  else {                                  // If SD card is not inserted
    if (logging){                         // ... and the card is newly removed
      logging = 0;                        // ... ... stop logging
      digitalWrite(greenLEDpin, LOW);     // ... ... turn green LED off
      Serial.print(" -- Card removed, stopped logging"); } // Message to user
    return 0;  }                          // ... no SD card, return 0
}

/***** Initialize Logfile ******/
void Logfile_init()
{
  Serial.println();
  Serial.println("Initializing SD card...");        // Initialize SD card
  if (!digitalRead(CDpin)){                       // If SD card is inserted
    if (!SD.begin(chipSelect)) {                  // Card is present and can be initialized?
      error("Card failed, or not present");       // ...not present or can't be initialized
    }
    else {                                        // ... card is present and can be initialized
      logging = 1;                                // ... now we are logging
      digitalWrite(greenLEDpin, HIGH);            // ... green LED on (logging to SD card)
      Serial.println("card initialized.");        // ... message to user
      for (uint8_t i = 0; i < 1000; i++) {        // Try a filename numbered 0-999
        filename[5] = i/100 + '0';
        filename[6] = i/10 + '0';
        filename[7] = i%10 + '0';
        if (! SD.exists(filename)) {               // If filename does not exist
          logFile = SD.open(filename, FILE_WRITE); // ...then use this filename
          Serial.print("Logging to: ");            // Message to user
          Serial.println(filename);    
          break;  }                                // .. and leave the loop
      }
    }
    if (! logFile) {                               // File created?
      error("couldnt create file");  }             // Error if no file created 
    logFile.close();                               // Close the logfile for now
    Serial.println();
  }
  rtc.timeNow();                                   // Read current time
  Log("\"Sensor number: ",SensorNumber,logFile); // Write time into header
  Log("\r\nDS1307 time: "+ rtc.dateString() + ", " + rtc.timeString(),0,logFile); // Write time into header
  Log("\r\nSHT21 sensor values (temperature2, humidity)",0,logFile);        // Write humidity sensor infor into log file
  Log("\r\nBMP180 sensor values (temperature, pressure)\"\r\n\r\n",0,logFile);      // Write pressure sensor infor into log file
  Log("  time (h), temp2 (C), hum (%RH), temp1 (C),  p (mbar),  time",0,logFile);                    // Write table header into log file
}

/******* Log data to Logfile ********/
void Log(String str, float num, File logfile)  // Log some text and a float number to logfile
{
  char tmp[12];                               // Helper variable for float -> String conversion
  if (num){                                   // Log numerical value only if not ==0 
    dtostrf(num,10,1,tmp);                     // Add numberical value to log string 
    str += tmp;                               // (width >=0, precision 1 digit)
  }
  if (SD_present()){                          // Check if card is available and ready
    digitalWrite(redLEDpin, HIGH);            // Red LED to indicate write (do not remove card)
    logfile = SD.open(filename, FILE_WRITE);  // Open file for write
    logfile.print(str);                       // Write log string into log file
    logfile.close();                          // Close log file
    digitalWrite(redLEDpin, LOW);             // Write done
  }
  else{logging = 0;}                          // If card not present, then logging=0
  if (ECHO_TO_SERIAL){                        // Write log string to Serial
    Serial.print(str);  
  }
}

/******* Error handling ********/
void error(char *str)
{
  Serial.print("error: ");
  Serial.println(str);
  digitalWrite(redLEDpin, HIGH);          // Red LED on    | 
  digitalWrite(greenLEDpin, LOW);         // Green LED off | indicates error
//  while(1);                               // Stop here
}
